angular.module('googleApi', [])
	.value('version', '0.1')

    .service("googleApiBuilder", function($q) {
        this.loadClientCallbacks = [];

        this.build = function(requestBuilder, responseTransformer) {
            return function(args) {
                var deferred = $q.defer();
                var response;
                request = requestBuilder(args);
                request.execute(function(resp, raw) {
                    if(resp.error) {
                        deferred.reject(resp.error);
                    } else {
                        response = responseTransformer ? responseTransformer(resp) : resp;
                        deferred.resolve(response);
                    }

                });
                return deferred.promise;

            };
        };

        this.afterClientLoaded = function(callback) {
            this.loadClientCallbacks.push(callback);
        };

        this.runClientLoadedCallbacks = function() {
            for(var i=0; i < this.loadClientCallbacks.length; i++) {
                this.loadClientCallbacks[i]();
            }
        };
    })

    .provider('googleLogin', function() {

        this.configure = function(conf) {
            this.config = conf;
        };

        this.$get = function ($q, googleApiBuilder, $rootScope) {
            var config = this.config;
            var deferred = $q.defer();
            return {
                login: function () {
                    gapi.auth.authorize({ client_id: config.clientId, scope: config.scopes, immediate: false}, this.handleAuthResult);

                    return deferred.promise;
                },

                handleClientLoad: function () {
                    gapi.auth.init(function () { });
                    window.setTimeout(checkAuth, 1);
                },

                checkAuth: function() {
                    gapi.auth.authorize({ client_id: config.clientId, scope: config.scopes, immediate: true }, this.handleAuthResult );
                },

                handleAuthResult: function(authResult) {
                    if (authResult && !authResult.error) {
                        var data = {};
                        $rootScope.$broadcast("google:authenticated", authResult);
                        googleApiBuilder.runClientLoadedCallbacks();
                        deferred.resolve(data);
                    } else {
                        deferred.reject(authResult.error);
                    }
                },
            };
        };


    })

    .service("googleCalendar", function(googleApiBuilder, $rootScope) {

        var self = this;
        var itemExtractor = function(resp) {
        return resp.items; };

        googleApiBuilder.afterClientLoaded(function() {
            gapi.client.load('calendar', 'v3', function() {
                self.listCalendars = function(callback){
                	var request = gapi.client.calendar.calendarList.list();
                	    request.then(function(resp) {                	    	
                	    	if(!resp.error) {
                                var calendar = [];
                                for(var i = 0; i < resp.result.items.length; i++) {
                                	item={};
                                    item.id=resp.result.items[i].id;
                                    item.summary=resp.result.items[i].summary;
                                    calendar.push(item);
                                }
                                callback(calendar);
                              }
                              else {
                                callback(resp.error);
                              }
                	    	});                	
                };
              
                self.listEvents =  function(id,callback){
                	var request = gapi.client.calendar.events.list({
                	      'calendarId': id,
                	      "timeMin":  new Date(new Date().getFullYear(), 0, 1).toISOString()
                	    });
                	request.then(function(resp) {  
                	//	alert(JSON.stringify(resp.result.items[0].start.date));
            	    	if(!resp.error) {
                            var calendar = [];
                            for(var i = 0; i < resp.result.items.length; i++) {
                            	item={};
                                item.id=resp.result.items[i].id;
                                item.summary=resp.result.items[i].summary;
                                if(resp.result.items[i].start.date){item.date=resp.result.items[i].start.date;}
                                if(resp.result.items[i].start.dateTime){
                                	var sd = new Date(resp.result.items[i].start.dateTime);
                                	var ed = new Date(resp.result.items[i].end.dateTime);
                                	item.dateTime=resp.result.items[i].start.dateTime.substring(0,10);
                                	item.startTime= pad(sd.getHours())+':'+ pad(sd.getMinutes())+':00';
                                	item.endTime= pad(ed.getHours())+':'+ pad(ed.getMinutes())+':00';                                	
                                }
                                calendar.push(item);
                            }
                            callback(calendar);
                          }
                          else {
                            callback(resp.error);
                          };
            	  });
                  function pad(d) {
            	        return (d < 10) ? '0' + d.toString() : d.toString();
            	  }
                }; 
            $rootScope.$broadcast("googleCalendar:loaded");
            });

        });

    });

		